package com.gree.mall.contest.controller.mini.coupon;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.coupon.CouponBean;
import com.gree.mall.contest.bean.coupon.CouponObtainBean;
import com.gree.mall.contest.bean.coupon.CouponQrcodeBean;
import com.gree.mall.contest.bean.coupon.CustomCoupouBean;
import com.gree.mall.contest.bean.user.UserCouponCountBean;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.coupon.CouponLogic;
import com.gree.mall.contest.logic.user.UserLogic;
import com.gree.mall.contest.plus.entity.UserCoupon;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Tag(name = "优惠券管理", description = "优惠券管理")
@RequestMapping(value = "/mini/coupon", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniCouponController {

    private final CouponLogic couponLogic;
    private final UserLogic userLogic;
    private final RedisLockRegistry redisLockRegistry;


    @GetMapping("/timeout")
    @Operation(summary = "优惠券过期提醒")
    public ResponseHelper<Boolean> timeout(
            @Parameter(description = "用户id") @RequestParam(required = false) String userId
    ) {
        Boolean timeout = couponLogic.timeout(userId);
        return ResponseHelper.success(timeout);
    }


    @GetMapping("/count")
    @Operation(summary = "优惠券统计")
    public ResponseHelper<UserCouponCountBean> count(
            @Parameter(description = "用户id") @RequestParam(required = false) String userId
    ) {
        UserCouponCountBean count = couponLogic.count(userId);
        return ResponseHelper.success(count);
    }

    @GetMapping("/list/page")
    @Operation(summary = "优惠券列表")
    public ResponseHelper<Page<UserCoupon>> page(
            @Parameter(description = "用户id") @RequestParam(required = false) String userId,
            @Parameter(description = "0:未使用  1:已使用 2:已过期") @RequestParam(required = false) Integer type,
            @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize

    ) throws RemoteServiceException {
        IPage<UserCoupon> page = couponLogic.miniPage(userId, type, pageNum, pageSize);
        return ResponseHelper.success(page);
    }

    @GetMapping("/list/expiring")
    @Operation(summary = "快要到期优惠券列表")
    public ResponseHelper<List<UserCoupon>> expiring(
            @Parameter(description = "用户id") @RequestParam(required = false) String userId
    ) throws RemoteServiceException {
        List<UserCoupon> coupons = couponLogic.expiring(userId);
        return ResponseHelper.success(coupons);
    }

    @GetMapping("/list/all")
    @Operation(summary = "可领/已领/待领优惠券列表")
    public ResponseHelper<List<CouponObtainBean>> listAll(
            @Parameter(description = "用户id") @RequestParam(required = false) String userId,
            HttpServletRequest request
    ) throws RemoteServiceException {
        List<CouponObtainBean> coupons = couponLogic.allObtainCoupou(userId, request);
        return ResponseHelper.success(coupons);
    }


    @GetMapping("/obtain")
    @Operation(summary = "领券")
    public ResponseHelper obtain(HttpServletRequest request,
                                 @Parameter(description = "用户id") @RequestParam(required = false) String userId,
                                 @Parameter(description = "优惠券id") @RequestParam(required = false) List<String> couponIds

    ) throws RemoteServiceException, InterruptedException {
        for (String couponId : couponIds) {
            Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_COUPON + couponId);
            if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
                log.error("系统繁忙，领券失败,userId:{},couponId:{}", userId, couponId);
                continue;
            }
            try {
                couponLogic.obtainCoupou(request, userId, couponId);
            } finally {
                obtain.unlock();
            }
        }
        return ResponseHelper.success();
    }


    @GetMapping("/list/useable")
    @Operation(summary = "可用优惠券列表")
    public ResponseHelper<List<CustomCoupouBean>> useableCoupou(
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId,
            @Parameter(description = "订单金额", required = true) @RequestParam(required = true) BigDecimal orderAmount,
            @Parameter(description = "商品ids", required = true) @RequestParam(required = true) List<String> goodsSpecIds
    ) throws RemoteServiceException {
        return ResponseHelper.success(couponLogic.useableCoupou(userId, orderAmount, goodsSpecIds));
    }


    @GetMapping("/detail")
    @Operation(summary = "券详情")
    public ResponseHelper<CouponBean> detail(
            @Parameter(description = "id", required = true) @RequestParam String couponId
    ) throws RemoteServiceException {
        UserCoupon detail = couponLogic.miniDetail(couponId);
        return ResponseHelper.success(detail);
    }

    @GetMapping("/transfer/qrcode")
    @Operation(summary = "获取转赠二维码")
    public ResponseHelper<CouponQrcodeBean> qrcode(@Parameter(description = "id", required = true) @RequestParam String userCouponId)
            throws RemoteServiceException {
        return ResponseHelper.success(couponLogic.qrcode(userCouponId));
    }


    @GetMapping("/transfer/coupon")
    @Operation(summary = "转赠优惠券")
    public ResponseHelper<String> transferCoupon(HttpServletRequest request, @Parameter(description = "id", required = true) @RequestParam String userCouponId)
            throws RemoteServiceException {

        return ResponseHelper.success(couponLogic.transferCoupon(request, userCouponId, userLogic));
    }


}
